import { Buffer } from './buffer.js'

/**
 * @typedef GeometryDescriptor
 * @param {Buffer} vertices
 * @param {number} [vertexCount]
 * @param {GPUVertexBufferLayout} layout
 * @property {Buffer} [indices]
 * @property {number} [indexCount]
 * @property {GPUIndexFormat} [format]
 */

export class Geometry {
  _device
  _vertices
  _vertexCount
  _vertexBufferLayout
  _indices
  _indexCount
  _format

  /**
   * @param {GPUDevice} device
   * @param {GeometryDescriptor} descriptor
   */
  constructor(device, descriptor) {
    this._device = device
    this._vertices = descriptor.vertices
    this._vertexCount = descriptor.vertexCount
    this._vertexBufferLayout = descriptor.layout

    if (descriptor.indices) {
      this._indices = descriptor.indices
      this._indexCount = descriptor.indexCount
      this._format = descriptor.format

      if (this._indexCount == null || this._format == null) {
        throw new Error('indexCount or format is required when creating an indexed geometry')
      }
    }
  }
}

