import { FlagEnum } from '../utils/index.js'

export const ShaderStage = FlagEnum(
  'vertex',
  'fragment',
  'compute'
)

export const stageFlagToName = (stages: GPUShaderStageFlags): string[] => {
  const names = []

  if (stages & GPUShaderStage.FRAGMENT) {
    names.push('fragment')
  }

  if (stages & GPUShaderStage.VERTEX) {
    names.push('vertex')
  }

  if (stages & GPUShaderStage.COMPUTE) {
    names.push('compute')
  }

  return names
}

export type StageFlag = 'fragment' | 'vertex' | 'compute'

export const nameToStageFlag = (names: StageFlag[]) => {
  return names.reduce((flags, name) => {
    switch (name.toLowerCase()) {
      case 'fragment': return flags | GPUShaderStage.FRAGMENT
      case 'vertex': return flags | GPUShaderStage.VERTEX
      case 'compute': return flags | GPUShaderStage.COMPUTE
    }
  }, 0)
}

export const ResourceType = Object.freeze({
  Sampler: 0,
  TextureView: 1,
  Buffer: 2,
  ExternalTexture: 3
})

